import { Bit } from '../types/bit';
import { Cell } from './cell';
import { Slice } from './slice';
import type { Address } from '../address';
import type { Coins } from '../coins';
import type { HashmapE } from './hashmap';
declare class Builder {
    private _size;
    private _refs;
    private _bits;
    constructor(size?: number);
    private checkBitsOverflow;
    private checkRefsOverflow;
    private storeNumber;
    get size(): number;
    get bits(): Bit[];
    get bytes(): Uint8Array;
    get refs(): Cell[];
    get remainder(): number;
    storeSlice(slice: Slice): this;
    storeRef(ref: Cell): this;
    storeBit(bit: Bit | number): this;
    storeBits(bits: Bit[]): this;
    storeInt(value: number | bigint, size: number): this;
    storeUint(value: number | bigint, size: number): this;
    storeVarInt(value: number | bigint, length: number): this;
    storeVarUint(value: number | bigint, length: number): this;
    storeBytes(value: Uint8Array | number[]): this;
    storeString(value: string): this;
    storeAddress(address: Address | null): this;
    storeCoins(coins: Coins): this;
    storeDict(hashmap: HashmapE<any, any>): this;
    clone(): Builder;
    cell(isExotic?: boolean): Cell;
}
export { Builder };
