"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Coins = void 0;
const decimal_js_1 = __importDefault(require("decimal.js"));
class Coins {
    constructor(value, isNano = false) {
        Coins.checkCoinsType(value);
        const nano = !isNano
            ? new decimal_js_1.default(value.toString()).mul(1e9)
            : new decimal_js_1.default(value.toString());
        this.value = nano;
    }
    add(value) {
        Coins.checkCoinsType(value);
        const nano = new decimal_js_1.default(value.toString()).mul(1e9);
        this.value = this.value.add(nano);
        return this;
    }
    sub(value) {
        Coins.checkCoinsType(value);
        const nano = new decimal_js_1.default(value.toString()).mul(1e9);
        this.value = this.value.sub(nano);
        return this;
    }
    mul(value) {
        Coins.checkCoinsType(value);
        const multiplier = value.toString();
        this.value = this.value.mul(multiplier);
        return this;
    }
    div(value) {
        Coins.checkCoinsType(value);
        const divider = new decimal_js_1.default(value.toString());
        this.value = this.value.div(divider);
        return this;
    }
    eq(value) {
        Coins.checkCoinsType(value);
        const nano = new decimal_js_1.default(value.toNano());
        return this.value.eq(nano);
    }
    gt(value) {
        Coins.checkCoinsType(value);
        const nano = new decimal_js_1.default(value.toNano());
        return this.value.gt(nano);
    }
    gte(value) {
        Coins.checkCoinsType(value);
        const nano = new decimal_js_1.default(value.toNano());
        return this.value.gte(nano);
    }
    lt(value) {
        Coins.checkCoinsType(value);
        const nano = new decimal_js_1.default(value.toNano());
        return this.value.lt(nano);
    }
    lte(value) {
        Coins.checkCoinsType(value);
        const nano = new decimal_js_1.default(value.toNano());
        return this.value.lte(nano);
    }
    isNegative() {
        return this.value.isNegative();
    }
    isPositive() {
        return this.value.isPositive();
    }
    isZero() {
        return this.value.isZero();
    }
    toString() {
        const value = this.value.div(1e9).toFixed(9);
        const coins = value.replace(/\.0{9}$/, '')
            .replace(/(\.\d*?[1-9])0+$/, '$1');
        return coins;
    }
    toNano() {
        return this.value.toFixed(0);
    }
    static checkCoinsType(coins) {
        if (!Coins.isValid(coins)) {
            throw new Error('Invalid coins value');
        }
    }
    static isValid(coins) {
        return coins instanceof Coins
            || typeof coins === 'string'
            || typeof coins === 'number'
            || typeof coins === 'bigint';
    }
    static fromNano(value) {
        Coins.checkCoinsType(value);
        return new Coins(value, true);
    }
}
exports.Coins = Coins;
//# sourceMappingURL=coins.js.map